# -*- coding: latin-1 -*-

"""

This is a helper class for finding free times for attendees of an event.

1) ---
2)     ---
3)   ---
4)          -----
5)             -----
            
>> -------  --------
"""


class MetaSpans:

    """
    Populate with a list of span tuples [(start,end)], and it will make "meta" 
    spans, with overlapping spans folded into one span.
    """

    def __init__(self, spans):
        self.spans = spans

    def get_spans(self):
        """
        Given a list of span tuples [(start,end)], will generate all meta spans, 
        with overlapping spans folded into one span.
        """
        spans = self.spans
        spans.sort()
        spans = iter(spans)
        metaspan = spans.next()
        for span in spans:
            start, end = span
            m_start, m_end = metaspan
            if start > m_end:
                yield metaspan
                metaspan = span
            elif end > m_end:
                metaspan = (m_start, end)
        # Need to yield the final metaspan once the span list is exhausted
        yield metaspan
    
    def non_spans(self):
        """
        Gets all the non-spans in between a sequence of metaspans
        """
        metaspans = iter(self.get_spans())
        _, prev_end = metaspans.next()
        for metaspan in metaspans:
            start, end = metaspan
            yield (prev_end, start)
            prev_end = end


if __name__ == '__main__':

    spans = [(0,3), (4,7), (2,5), (9,14), (12,17)]
    ms = MetaSpans(spans)
    print 'Spans:', spans
    print list(ms.get_spans())
    print list(ms.non_spans())
    print ''

    spans = [(5,10), (6,7), (3,15)]
    ms = MetaSpans(spans)
    print 'Spans:', spans
    print list(ms.get_spans())
    print list(ms.non_spans())
    print ''

    from datetime import datetime
    cal_spans = [
        (datetime(2005, 1, 1, 0, 0, 0), datetime(2005, 1, 1, 3, 0, 0)),
        (datetime(2005, 1, 1, 4, 0, 0), datetime(2005, 1, 1, 7, 0, 0)),
        (datetime(2005, 1, 1, 2, 0, 0), datetime(2005, 1, 1, 5, 0, 0)),
        (datetime(2005, 1, 1, 9, 0, 0), datetime(2005, 1, 1, 14, 0, 0)),
        (datetime(2005, 1, 1, 12, 0, 0), datetime(2005, 1, 1, 17, 0, 0)),
    ]
    ms = MetaSpans(cal_spans)
    print 'Spans:', spans
    print list(ms.get_spans())
    print list(ms.non_spans())
    print ''
    

