# python
import os, os.path

# cmf
from Products.CMFCore.utils import getToolByName

PRODUCT_ID = 'mxmCalendarTypes'

def tupelize(st):
    "Converts version string to tuple of ints"
    return tuple(map(int, st.split('.')))

def strize(tp):
    "Converts version tuple to string"
    return '.'.join(map(str, tp))


class Migrator:

    ###############################
    # Migration framework start

    def __init__(self, context):
        self.context = context
        self._n_migrated = 0

    def getPortal(self):
        context = self.context
        return getToolByName(context, 'portal_url').getPortalObject()

    def getProductFile(self, fname='readme.txt'):
        """
        Return the content of a file of the product or None. case-insensitive.
        """
        context = self.context
        try:
            prodpath=context.Control_Panel.Products._getOb(PRODUCT_ID).home
        except AttributeError:
            return None
        #now list the directory to get the readme.txt case-insensitive
        try:
            files=os.listdir(prodpath)
        except OSError:
            return None
        for f in files:
            if f.lower()==fname:
                return open(os.path.join(prodpath,f)).read()
        return None


    def getProductVersion(self):
        "Return the version string stored in version.txt"
        context = self.context
        quickinstaller = getToolByName(context, 'portal_quickinstaller')
        installedProduct = quickinstaller._getOb(PRODUCT_ID, None)
        if not installedProduct is None:
            version_txt = installedProduct.getInstalledVersion()
            return tupelize(version_txt)

    # The main migration method.

    def migrate(self, out):
        "The main migration method"
        productVersion = self.getProductVersion()
        if productVersion is None:
            print >>out, 'Could not find product version. Migration stopped.'
            return
        print >>out, 'productVersion', strize(productVersion)
        #
        for version, migratorName in MIGRATORS:
            if version > productVersion:
                print >>out, 'Migrating to', strize(version)
                migrator = getattr(self, migratorName)
                migrator(out)
            else:
                print >>out, 'Skipped:', strize(version)
        print >>out, 'Done! %s objects needed upgrading to current version.' % self._n_migrated
        # XXX abort while debugging ...

    # Migration framework end
    ###############################



    ######################################
    # Migration method(s) start

    # to 2.1.0

    MEMBER_ID = 'X-MXM-MEMBER-ID'
    USED_TIME = 'X-MXM-USED-TIME'

    def updateTimreg_2_1_0(self, obj, path=''):
        "Updates the mxmTodo objects"
        print 'found a todo', obj.title_or_id()
        attendees = getattr(obj, '_attendees', None)
        if attendees is None:
            return
        for id, att in attendees.items():
            used_time = att.get(self.USED_TIME, None)
            if not used_time is None:
                memberId = att.get(self.MEMBER_ID, None)
                if not memberId is None:
                    print 'upgrading it', att
                    print 'time', used_time
                    obj.addTimeRegistration(memberId, time=used_time)
                    print obj.getTimeRegsByKeyGroup('memberId')
                    del att[self.USED_TIME]
                    self._n_migrated += 1


    def migrate_to_2_1_0(self, out):
        "Migrates from previous versions to 2.1.0"
        context = self.context
        print >>out, "Converting old time registrations to new"
        portal = self.getPortal()
        context.ZopeFindAndApply(portal, obj_metatypes=['mxmTodo'], 
                                 search_sub=1,
                                 apply_func=self.updateTimreg_2_1_0)

    # next version here ...

    # Migration method(s) end
    ######################################



# am ordered sequence that maps a version tuple to a migration method name
MIGRATORS = (
    ( (2,1,0),'migrate_to_2_1_0' ),
)

