# -*- coding: ISO-8859-1 -*-

# python
import urlparse, smtplib
from datetime import datetime, timedelta
from sets import Set
from types import ListType, StringTypes
from email.Message import Message
from email.MIMEMultipart import MIMEMultipart
from email.Header import Header
from email import Encoders
from math import modf

# Zope
from Products.ZCatalog.ZCatalog import ZCatalog
from AccessControl import ClassSecurityInfo
from AccessControl import Unauthorized
from DateTime import DateTime

# cmf
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import View, ListFolderContents
from Products.CMFCore.CMFCorePermissions import setDefaultRoles
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.DiscussionTool import DiscussionNotAllowed

# Plone
from Products.CMFPlone import ToolNames # for getting comments

# Archetypes
from Products.Archetypes.public import BaseContent, BaseFolder, registerType
from Products.Archetypes.public import DisplayList

try:
    # iCalendar
    from icalendar import Calendar, Event, Todo, Journal, LocalTimezone
    from icalendar import ComponentFactory, TypesFactory, vText, vCalAddress
    vTypes = TypesFactory()
    HAS_ICAL = True
except:
    HAS_ICAL = False

# product
from config import PROJECTNAME, VIEW_CONTENTS_PERMISSION,\
                   EDIT_CONTENTS_PERMISSION, ADD_CONTENTS_PERMISSION
from schemas import mxmTodo_schema, mxmJournal_schema, mxmEvent_schema
from MetaSpans import MetaSpans

def upperKeys(dct):
    "Returns copy of dct, but with keys upper case"
    upper = {}
    for key, val in dct.items():
        upper[key.upper()] = val
    return upper


def dt2DT(dt):
    "Converts Python datetime to Zope DateTime"
    try:
        # convert dt to local timezone
        ltz = dt.astimezone(LocalTimezone())
        return DateTime(*ltz.timetuple()[:6])
    except:
        return DateTime(dt.year, dt.month, dt.day, 0, 0, 0)


def DT2dt(DT):
    """
    Converts Zope DateTime to Python datetime, Zope DateTime is allways utc
    """
    return datetime.fromtimestamp(DT.timeTime(), LocalTimezone())


# shared by all components
common_properties =  [
    'attendee', 'categories', 'class', 'comment', 'contact', 'created', 
    'description', 'dtstamp', 'dtstart', 'last_modified', 'organizer', 
    'sequence', 'status', 'summary', 'uid', 'url',
]

# in individual components
component_properties = {
    'VEVENT'   : ['dtend', 'location', 'priority', 'transp'],
    'VTODO'    : ['completed', 'due', 'dtend', 'location', 'percent', 'priority'],
    'VJOURNAL' : [],
}

class ComponentSetter:

    """
    Populates an iCalendar component from a Calendar type
    """

    def __init__(self, componentName, charset):
        self.componentName = componentName.upper()
        self.charset = charset

    def getProperties(self):
        # returns propeties for component
        props = common_properties + \
                component_properties.get(self.componentName, ())
        props.sort()
        return props

    def dec(self, text): # decode to unicode
        return text.replace('\r\n', '\n').decode(self.charset)

    # public method
    def populate(self, obj):
        "Converts event to ical vEvent component"
        self.obj = obj
        factory = ComponentFactory()
        self.component = factory[self.componentName]()
        # populate the component
        for propertyName in self.getProperties():
            getattr(self, 'set_%s' % propertyName)()
        return self.component

    def set_text(self, name, text):
        if text: self.component.add(name, self.dec(text))

    def set_datetime(self, name, dt):
        if dt: self.component.add(name, DT2dt(dt))

    # setters

    def set_attendee(self):
        # attendees
        atts = []
        attendees = self.obj.listAttendees()
        for id, attendee in attendees:
            email = attendee.get(EMAIL_KEY, '')
            if email:
                att = vCalAddress('MAILTO:' + email)
                for key, val in attendee.items():
                    att.params[str(key)] = str(val)
                #att.params.update(attendee)
                att.params[ATTENDEE_IDX] = str(id)
                atts.append(att)
        if atts:
            self.component['attendee'] = atts

    def set_categories(self):
        # these are shared special properties for all components categories
        val = self.obj.Subject()
        if val:
            categories = [self.dec(s) for s in val]
            self.component.set_inline('subject', categories)

    def set_class(self):
        self.set_text('class', self.obj.getClass())

    def set_comment(self):
        # we only export the first level of comments
        for comment in self.obj.getComments():
            self.set_text('comment', comment)

    def set_completed(self):
        if not self.obj.getCompleted is None:
            self.set_datetime('completed', self.obj.getCompleted())

    def set_contact(self):
        self.set_text('contact', self.obj.contact_phone())

    def set_created(self):
        if not self.obj.created is None:
            self.set_datetime('created', self.obj.created())

    def set_description(self):
        self.set_text('description', self.obj.Description())

    def set_dtend(self):
        if not self.obj.end is None:
            self.set_datetime('dtend', self.obj.end())

    def set_dtstamp(self):
        self.set_datetime('dtstamp', self.obj.getDTStamp())

    def set_dtstart(self):
        if not self.obj.start is None:
            self.set_datetime('dtstart', self.obj.start())

    def set_due(self):
        if not self.obj.end is None:
            self.set_datetime('due', self.obj.end())

    def set_last_modified(self):
        self.set_datetime('last-modified', self.obj.modified())

    def set_location(self):
        self.set_text('location', self.obj.getLocation())

    def set_organizer(self):
        # organizer
        obj = self.obj
        name = self.dec(obj.contact_name())
        mail = self.dec(obj.contact_email())
        role = self.dec(obj.getContactRole())
        if mail:
            self.component.add('organizer', u'MAILTO:' + mail)
            if name:
                self.component['organizer'].params['cn'] = vText(name).ical()
            if role:
                self.component['organizer'].params['role'] = vText(role).ical()

    def set_percent(self):
        self.component.add('percent-complete', self.obj.getPercent())

    def set_priority(self):
        self.component.add('priority', self.obj.getPriority())

    def set_sequence(self):
        self.component.add('sequence', self.obj.getSequence())

    def set_status(self):
        self.set_text('status', self.obj.getStatus())

    def set_summary(self):
        self.set_text('summary', self.obj.title_or_id())

    def set_transp(self):
        self.set_text('transp', self.obj.getTransparency())

    def set_uid(self):
        self.set_text('uid', self.obj.getIcalUid())

    def set_url(self):
        self.set_text('url', self.obj.getUrl())



class EventSetter:

    """
    Populates a calendar type from an iCalendar component
    """

    def __init__(self, componentName, charset):
        self.componentName = componentName.upper()
        self.charset = charset

    def getProperties(self):
        # returns propeties for component
        props = common_properties + \
                component_properties.get(self.componentName, ())
        props.sort()
        return props

    def enc(self, text): # encode to local charset
        return text.encode(self.charset)

    def dec(self, text): # decode to unicode
        return text.replace('\r\n', '\n').decode(self.charset)

    # public method
    def populate(self, obj, component):
        "populates event from ical vEvent component"
        self.obj = obj
        self.component = component
        # populate the component
        for propertyName in self.getProperties():
            getattr(self, 'set_%s' % propertyName)()
        self.obj.reindexObject()
        return self.obj

    def set_text(self, name, setter):
        val = self.component.decoded(name, None)
        if val: setter(self.enc(val))

    def set_datetime(self, name, setter):
        val = self.component.decoded(name, None)
        if val: setter(dt2DT(val))
    
    # setters

    def set_attendee(self):
        component = self.component
        obj = self.obj
        LEN_MTO = 7
        if 'ATTENDEE' in component:
            attendees = component['ATTENDEE']
            if not type(attendees) is ListType:
                attendees = [attendees]
            for attendee in attendees:
                new_attendee = {} # we save attendees as dicts
                params = attendee.params
                attendee_email = self.enc(vText.from_ical(attendee))[LEN_MTO:]
                new_attendee[EMAIL_KEY] = attendee_email
                for key, val in params.items():
                    new_attendee[key] = self.enc(vText.from_ical(val))
                obj._attendees.append(new_attendee)

    def set_categories(self):
        if 'categories' in self.component:
            subject = self.component.get_inline('categories')
            self.obj.setSubject([self.enc(s) for s in subject])

    def set_class(self):
        # class is a property of workflow,so we dont set it
        pass

    def set_comment(self):
        # only exporting supported so far ..
        pass

    def set_completed(self):
        self.set_datetime('completed', self.obj.setCompleted)

    def set_contact(self):
        self.set_text('contact', self.obj.setContact)

    def set_created(self):
        pass # this is set by zope

    def set_description(self):
        self.set_text('description', self.obj.setDescription)

    def set_dtend(self):
        if 'dtend' in self.component:
            self.set_datetime('dtend', self.obj.setEndDate)

    def set_dtstamp(self):
        self.set_datetime('dtstamp', self.obj.setDTStamp)

    def set_dtstart(self):
        self.set_datetime('dtstart', self.obj.setStartDate)

    def set_due(self):
        # eg. GroupWise does not use due but dtend despite of spec :-s
        # So if there is no due but a dtend, we use that instead
        if 'due' in self.component:
            self.set_datetime('due', self.obj.setEnd)
        else:
            self.set_dtend()

    def set_last_modified(self):
        pass # set by zope

    def set_location(self):
        self.set_text('location', self.obj.setLocation)

    def set_organizer(self):
        # organizer
        LEN_MTO = 7
        component = self.component
        obj = self.obj
        if 'organizer' in component:
            email = component.decoded('organizer')
            obj.setContactEmail(self.enc(email[LEN_MTO:]))
            params = component['organizer'].params
            name = self.dec(params.get('cn', ''))
            if name:
                obj.setContactName(self.enc(name))
            role = self.dec(params.get('role', ''))
            if role:
                obj.setContactRole(self.enc(role))

    def set_percent(self):
        self.obj.setPercent(self.component.decoded('percent-complete', 0))

    def set_priority(self):
        self.obj.setPriority(self.component.decoded('priority', 0))

    def set_sequence(self):
        self.obj.setSequence(self.component.decoded('sequence', 0))

    def set_status(self):
        self.set_text('status', self.obj.setStatus)

    def set_summary(self):
        self.set_text('summary', self.obj.setTitle)

    def set_transp(self):
        self.set_text('transp', self.obj.setTransparency)

    def set_uid(self):
        self.set_text('uid', self.obj.setIcalUid)

    def set_url(self):
        self.set_text('url', self.obj.setEventUrl)



class AutoDict(dict):

    """
    A dictionary with autoincrementing integer keys.
    """

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, *args, **kwargs):
        dict.__init__(self, *args, **kwargs)
        self.__counter = 0

    def append(self, obj):
        key = self.__counter
        self[key] = obj
        self.__counter += 1
        return key


class UnrestrictedCatalog:

    """
    Small wrapper that allows unrestricted searches in the catalog.
    """

    def __init__(self, catalog):
        self.catalog = catalog

    # unbound method for unrestricted catalog searches
    def searchResults(self, REQUEST=None, **kw):
        "Unrestricted catalog search"
        return apply(ZCatalog.searchResults, (self.catalog, REQUEST), kw)

    __call__ = searchResults


# constants

EMAIL_KEY = 'X-MXM-EMAIL'
MEMBER_KEY = 'X-MXM-MEMBER-ID'
ATTENDEE_IDX = 'X-MXM-ATT-IDX'
USED_TIME_KEY = 'X-MXM-USED-TIME'

# DateTime constants

DAY = 1.0
HOUR = DAY / 24.0
MINUTE = HOUR / 60.0

DOW_MAP = {'sunday':0,'monday':1,'tuesday':2,'wednesday':3,'thursday':4,'friday':5, 'saturday':6}

class mxmCalendarType(BaseContent):

    content_icon = 'event_icon.gif'

    _at_rename_after_creation = True
    
    # list of icalendar properties that the calendar type implements.
    implemented_ical_properties = Set()

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(VIEW_CONTENTS_PERMISSION)

    # 2.1 tyle id from title allowed
    _at_rename_after_creation = True
    
#    security.declarePublic('test')
#    def test(self):
#        "Test method"
#        meta = getToolByName(self, 'portal_metadata')
#        subject_spec = meta.getElementSpec('Subject')
#        if not subject_spec is None:
#            return repr(subject_spec.getPolicy(self.content_type))

    security.declarePrivate('at_post_create_script')
    def at_post_create_script(self):
        "AT after creation hook"
        self.sequence = 0
        self._content_lines = []
        self._attendees = AutoDict()
        # call the initial attendees script.
        if not self._attendees and hasattr(self, 'mxmCalendarTypes_initialAttendees'):
            for attendee in self.mxmCalendarTypes_initialAttendees():
                self._attendees.append(attendee)

    security.declarePrivate('at_post_edit_script')
    def at_post_edit_script(self):
        "AT after edit hook"
        self.sequence += 1
        # a hook for changing the values after form processing
        # eg. to limit date ranges etc.
        if hasattr(self, 'mxmCalendarTypes_postEditHook'):
            self.mxmCalendarTypes_postEditHook()

    def charset(self):
        prop_tool = getToolByName(self, 'portal_properties')
        return prop_tool.site_properties.default_charset


    ####################################
    # Plumbing for iCalendar integration


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'hasIcal')
    def hasIcal(self):
        "Returns true if the icalendar package is installed."
        return HAS_ICAL


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getIcalUid')
    def getIcalUid(self):
        "Returns a globally unique id for the event"
        if hasattr(self, 'ical_uid'):
            return self.ical_uid
        else:
            netloc = urlparse.urlparse(self.absolute_url())[1].split(':')[0]
            return '%s@%s' % (self.UID(), netloc)


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'setIcalUid')
    def setIcalUid(self, uid):
        "Set a globally unique id for the event"
        self.ical_uid = uid


    # "PUBLIC" / "PRIVATE" / "CONFIDENTIAL" / x-name
    security.declarePublic('getClass')
    def getClass(self):
        "The class is based on the workflow state"
        workflow = getToolByName(self, 'portal_workflow')
        state = workflow.getInfoFor(self, 'review_state').upper()
        if state in ('VISIBLE', 'PUBLIC'):
            return 'PUBLIC'
        elif state == 'PRIVATE':
            return 'PRIVATE'
        else:
            return 'X-%s' % state

    # the sequence number is incremented by each edit

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getSequence')
    def getSequence(self):
        "Get zero base version number."
        return self.sequence

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'setSequence')
    def setSequence(self, sequence):
        "Set version number"
        self.sequence = sequence


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getUrl')
    def getUrl(self):
        "Returns the url of the event"
        event_url = self.event_url()
        if event_url:
            return event_url
        else:
            return self.absolute_url()

    def getDTStamp(self):
        "Returns the dstamp"
        return getattr(self, 'dtstamp', self.created())

    def setDTStamp(self, dtstamp):
        "Returns the dstamp"
        self.dtstamp = dtstamp

    # iCal supports comments, but I have never seen any app that uses it. Hooray, a first!
    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getComments')
    def getComments(self):
        "Returns list of comments"
        discuss_tool = getToolByName(self, 'portal_discussion')
        # we only return comments if it's the default discussion tool
        # other tools can have different interfaces
        comments = []
        if ToolNames.DiscussionTool == discuss_tool.meta_type:
            try:
                discussion = discuss_tool.getDiscussionFor(self)
                replies = discussion.getReplies()
                for reply in replies:
                    r = []
                    if reply.title:
                        r.append(reply.title)
                    if reply.text:
                        r.append(reply.text)
                    comments.append('\n'.join(r))
            except DiscussionNotAllowed:
                pass
        return comments


    ##############################
    # helpers for rendering events

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'event_anchor')
    def event_anchor(self):
        "Renders the event anchor"
        event_url = self.event_url()
        if event_url:
            return '<a href="%s">%s</a>' % (event_url, event_url)
        return ''

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'contact_anchor')
    def contact_anchor(self):
        "Renders the event anchor"
        contactName = self.contact_name()
        contactEmail = self.contact_email()
        if contactName and contactEmail:
            return '<a href="MAILTO:%s">%s</a>' % (contactEmail, contactName)
        elif contactEmail:
            return '<a href="MAILTO:%s">%s</a>' % (contactEmail, contactEmail)
        elif contactName:
            return contactName
        return ''



    security.declarePrivate('getAuthenticated')
    def getAuthenticated(self):
        "Returns authenticated member"
        return getToolByName(self, 'portal_membership').getAuthenticatedMember()

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getAuthenticatedsId')
    def getAuthenticatedsId(self):
        "Returns id of currently authenticated member."
        member = self.getAuthenticated()
        if member:
            return member.getMemberId()
        return ''

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getAuthenticatedsFullname')
    def getAuthenticatedsFullname(self):
        "Returns fullname of currently authenticated member"
        member = self.getAuthenticated()
        if member:
            return member.getProperty('fullname', '')
        return ''

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getAuthenticatedsEmail')
    def getAuthenticatedsEmail(self):
        "Returns email of currently authenticated member."
        member = self.getAuthenticated()
        if member:
            return member.getProperty('email', '')
        return ''

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getNow')
    def getNow(self):
        "Returns the time right now"
        return self.ZopeTime()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getInAnHour')
    def getInAnHour(self):
        "Returns the time an hour from now"
        return self.getNow() + HOUR

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'dateTimeView')
    def dateTimeView(self, date_format='%d/%m/%Y', time_format='%H:%M'):
        "Returns a nicely formatted datetime view"
        now = self.getNow()
        start = self.start() or now
        end = self.end() or now
        return self.dateTimeFormat(start, end, date_format, time_format)


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'dateTimeFormat')
    def dateTimeFormat(self, start, end, date_format='%d/%m/%Y', time_format='%H:%M'):
        "Returns a nicely formatted datetime view"
        date_time = (date_format, time_format)
        if start.strftime(date_format) == end.strftime(date_format):
            if start.strftime(time_format) == end.strftime(time_format):
                return start.strftime('%s %s' % date_time)
            return start.strftime('%s (%s-' % date_time) + end.strftime('%s)' % time_format)
        return '%s - %s' % (start.strftime('%s (%s)' % date_time), end.strftime('%s (%s)' % date_time))

    # iCalendar stuff

##    """
##    
##    if you populate a calendar type from an iCalendar component, there is a 
##    lot of properties that cannot be used/stored directly in a calendar type. To 
##    avoid data loss, this "rest" is just stored as a list of content lines 
##    (strings) on the object. They can then be added back when the object is 
##    converted to an iCal Component.
##    
##    This makes it possible to use the calendar types as a database for iCalendar 
##    files. Even using properties that are not defined in these types.
##    
##    """
##
##
##    def setContentLines(self, component):
##        "Gets the unimplemented properties from the component"
##        implemented = implemented_ical_properties + Set(['BEGIN', 'END'])
##        for propertyName in component.items():
##            if not propertyName in implemented:
##                l = str(component.content_line(propertyName))
##                self._content_lines.append(l)
#


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getIcalComponent')
    def getIcalComponent(self, setter=None):
        "Converts to ical component"
        if setter is None:
            return ComponentSetter(self.ical_type, self.charset()).populate(self)
        else:
            return setter.populate(self)


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'setIcalComponent')
    def setIcalComponent(self, component, setter=None):
        "populates from ical component"
        if setter is None:
            return EventSetter(self.ical_type, self.charset()).populate(self, component)
        else:
            return setter.populate(self, component)


    security.declarePrivate('_ics')
    def _ics(self):
        "Returns object as iCal string"
        cal = Calendar() # iCalendar object
        # properties required by rfc 2445
        netloc = urlparse.urlparse(self.absolute_url())[1].split(':')[0]
        cal.add('prodid', '-//mxmCalendarTypes//NOSGML %s//EN' % netloc)
        cal.add('version', '2.0')
        component = self.getIcalComponent()
        cal.add_component(component)
        return cal.as_string()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'ics')
    def ics(self):
        "Returns ical file"
        result = self._ics()
        # generate headers
        setHeader = self.REQUEST.RESPONSE.setHeader
        setHeader('Content-Length', len(result))
        setHeader('Content-Type', 'text/calendar; charset=UTF-8')
        setHeader('Content-Disposition', 'filename=%s.ics' % self.getId())
        return result


    ##############################3
    # Attendees


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAttendees')
    def getAttendees(self):
        "Returns list of attendees"
        return getattr(self, '_attendees', AutoDict())


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'setAttendees')
    def setAttendees(self, attendees):
        "Sets the list of attendees"
        self._attendees = attendees
        self._p_changed = 1


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'addAttendee')
    def addAttendee(self, member_id='', email='', name='', 
                    role='REQ-PARTICIPANT', status='TENTATIVE', 
                    used_time=None, REQUEST=None):
        """
        An attendee is basically a dict with the same keys as icalender 
        parameters. But with the keys X-MXM-MEMBER-ID and X-MXM-EMAIL added. 
        X-MXM-EMAIL is the same as the iCalandar property value.
        Any valid iCal parameter key:value can be expected in an attendee.
        """
        attendee = {}
        if member_id or email: # only a valid attendee with one of these
            if member_id:
                attendee[MEMBER_KEY] = member_id
            if email:
                attendee[EMAIL_KEY] = email
            if not used_time is None:
                attendee[USED_TIME_KEY] = used_time
            if name:
                attendee['CN'] = name
            attendee['ROLE'] = role
            attendee['PARTSTAT'] = status
            self._attendees.append(attendee)
            self._p_changed = 1
        if not REQUEST is None:
            self.REQUEST.RESPONSE.redirect('%s/mxmCalendarTypes_attendees_form' % self.absolute_url())


    def _setAttendeVals(self, attendee, req_key, att_key, REQUEST):
        val = REQUEST.get(req_key, '')
        if val:
            attendee[att_key] = val
        else:
            if att_key in attendee:
                del attendee[att_key]


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'editAttendees')
    def editAttendees(self, REQUEST=None):
        "Edit state for all attendees"
        for key, attendee in self.getAttendees().items():
            self._setAttendeVals(attendee, 'email_%s' % key, EMAIL_KEY, REQUEST)
            self._setAttendeVals(attendee, 'name_%s' % key, 'CN', REQUEST)
            self._setAttendeVals(attendee, 'role_%s' % key, 'ROLE', REQUEST)
            self._setAttendeVals(attendee, 'status_%s' % key, 'PARTSTAT', REQUEST)
            self._setAttendeVals(attendee, 'used_time_%s' % key, USED_TIME_KEY, REQUEST)
        self._p_changed = 1
        self.notifiyStatusChange()
        self.reindexObject()
        if not REQUEST is None:
            self.REQUEST.RESPONSE.redirect('%s/mxmCalendarTypes_attendees_form' % self.absolute_url())


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'delAttendees')
    def delAttendees(self, ids=None, REQUEST=None):
        "Delete list of attendees"
        if not ids is None:
            for id in ids:
                del self._attendees[id]
            self._p_changed = 1
        if not REQUEST is None:
            self.REQUEST.RESPONSE.redirect('%s/mxmCalendarTypes_attendees_form' % self.absolute_url())


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'attendee_ids')
    def attendee_ids(self):
        "Returns a list of memberids for the attendees of the event"
        attendees = self.getAttendees().values()
        result = []
        for attendee in attendees:
            id = attendee.get(MEMBER_KEY, '')
            if id:
                result.append(id)
        return result

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'nAttendees')
    def nAttendees(self):
        "Returns number of attendees on the task"
        return len(self.getAttendees())

    #

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getRolesMap')
    def getRolesMap(self):
        return {
            '' : 'Undefined',
            'CHAIR' : 'Chair',
            'REQ-PARTICIPANT' : 'Required participant',
            'OPT-PARTICIPANT' : 'Optional participant',
            'NON-PARTICIPANT' : 'Non participant',
        }

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'possibleRolesItems')
    def possibleRolesItems(self):
        "A list of possible roles"
        return [('', 'Undefined'), ('CHAIR', 'Chair'), 
                ('REQ-PARTICIPANT', 'Required participant'), 
                ('OPT-PARTICIPANT', 'Optional participant'),
                ('NON-PARTICIPANT', 'Non participant'), ]


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'listAttendees')
    def listAttendees(self):
        """
        Returns a list of dicts. Same as attendee, but with emails and names 
        populated from member data if relevant.
        """
        # XXX probably need to filter unlisted members out
        mtool = getToolByName(self, 'portal_membership')
        attendees = self.getAttendees().items()
        result = []
        for key, attendee in attendees:
            member_id = attendee.get(MEMBER_KEY, '')
            member_data = {}
            member_data.update(attendee)
            if member_id:
                member = mtool.getMemberById(member_id)
                if member:
                    member_data['CN'] = member.getProperty('fullname', '')
                    member_data[EMAIL_KEY] = member.getProperty('email', '')
            # overwrite with attende and fill in blanks
            result.append((key, member_data))
        return result


    security.declarePublic('changeOwnStatus')
    def changeOwnStatus(self, status, idx, used_time=None, REQUEST=None):
        """
        Change the status of the currently logged in member. It is public, so we 
        check that it's the right user calling it. It's there so that members 
        that does not have permission to edit the event can change their 
        participation status anyway.
        """
        attendee = self._attendees.get(idx, {})
        if attendee:
            mtool = getToolByName(self, 'portal_membership')
            member = mtool.getAuthenticatedMember()
            if member and (attendee[MEMBER_KEY] == member.getMemberId()):
                status_changed = (attendee['PARTSTAT'] != status)
                attendee['PARTSTAT'] = status
                if not used_time is None:
                    attendee[USED_TIME_KEY] = used_time
                self._p_changed = 1
                if status_changed:
                    self.notifiyStatusChange()
        if not REQUEST is None:
            self.REQUEST.RESPONSE.redirect(self.absolute_url())


    def _allStatusAccepted(self):
        "Returns true if all attendees status are set to accepted"
        allAccepted = 1
        for key, attendee in self.getAttendees().items():
            partstat = upperKeys(attendee).get('PARTSTAT', '')
            if partstat.upper() != 'ACCEPTED':
                allAccepted = 0
        return allAccepted


    def _allStatusSet(self):
        "Returns true if all attendees status are set/different than tentative"
        allSet = 1
        for key, attendee in self.getAttendees().items():
            partstat = upperKeys(attendee).get('PARTSTAT', '')
            if partstat.upper() in ['TENTATIVE', '']:
                allSet = 0
        return allSet


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'attendeeStatus')
    def attendeeStatus(self):
        "Returns list of uppercase dict, of attende status"
        return [upperKeys(attendee) for attendee in self.getAttendees().values()]


#    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getUsedTime')
#    def getUsedTime(self):
#        "Returns the time in hours, used by all members on an event"
#        attendees = self.getAttendees().values()
#        return sum([float(a.get(USED_TIME_KEY, 0.0)) for a in attendees])
#
#
#    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getUsedTimeForMember')
#    def getUsedTimeForMember(self, memberId):
#        "Returns the time in hours, used by a member on an event"
#        for att in self.getAttendees().values():
#            if att[MEMBER_KEY] == memberId:
#                return attendee.get(USED_TIME_KEY, 0.0)
#        return 0.0


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'timePerAttendee')
    def timePerAttendee(self):
        "Returns the average time per attendee"
        nAttendees = self.nAttendees()
        if nAttendees:
            return (self.getAllocatedTime() / nAttendees)
        else:
            return 0.0


    #######

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'listMemberIds')
    def listMemberIds(self):
        "list of member ids caselessly sorted"
        mtool = getToolByName(self, 'portal_membership')
        memberIds = mtool.listMemberIds()
        decorated = [(memberId.lower(), memberId) for memberId in memberIds]
        decorated.sort()
        return [m[-1] for m in decorated]


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getRooster')
    def getRooster(self):
        "caseless sorted list of [(memberId, fullname)]"
        mtool = getToolByName(self, 'portal_membership')
        members = mtool.searchForMembers(self.REQUEST)
        decorated = []
        for member in members:
            memberId = member.getMemberId()
            fullname = member.getProperty('fullname', '')
            decorated.append( (fullname.lower(), (memberId, fullname)))
        decorated.sort()
        return [m[-1] for m in decorated]


    ##########


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'allowedSubjects')
    def allowedSubjects(self):
        "Returns a list of subjects/categories that the user can select for this event"
        metadata_tool = getToolByName(self, 'portal_metadata')
        subjects = metadata_tool.listAllowedSubjects(self)
        return subjects


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'allowedSubjectsDisplayList')
    def allowedSubjectsDisplayList(self):
        "Returns a displaylist of allowedSubjects()"
        return DisplayList([(kw, kw) for kw in self.allowedSubjects()])

    def _site_encoding(self):
        "Returns the site encoding"
        portal_properties = getToolByName(self, 'portal_properties')    
        site_props = portal_properties.site_properties
        return site_props.default_charset or 'utf-8'

    def _site_email(self):
        "Returns the site email"
        portal = getToolByName(self, 'portal_url').getPortalObject()
        return portal.email_from_address

    def _mailhost(self):
        # tries to fetch a mailhost/smtp server url from several locations
        # try site setting
        if hasattr(self, 'MailHost'):
            mh = self.MailHost
            mailhost = mh.smtp_host
            if mailhost:
                return mailhost
        # desperately try sending via localhost
        return 'localhost'

### is this thing on? XXX
    def notifiyStatusChange(self):
        """
        This method is called when an events status is changed
        """
        if hasattr(self, 'mxmCalendarTypes_notifications'):
            organizer = self.contact_email()
            attendees = [att for (idx, att) in self.listAttendees()]
            if hasattr(self, 'mxmCalendarTypes_notifications'):
                to, subject, body = self.mxmCalendarTypes_notifications(organizer, attendees)
                if to:
                    self.sendMessage(to, subject, body)


    def notifiyAttendees(self, REQUEST=None):
        """
        This method is called when an events status is changed
        """
        attendees = [att for (idx, att) in self.listAttendees()]
        if hasattr(self, 'mxmCalendarTypes_notifiyAttendees'):
            to, subject, body = self.mxmCalendarTypes_notifiyAttendees(attendees)
            if to:
                self.sendMessage(to, subject, body)
        if not REQUEST is None:
            REQUEST.RESPONSE.redirect(self.absolute_url())


    security.declarePrivate('sendMessage')
    def sendMessage(self, to=None, subject='', body='', attachment=None, REQUEST=None):
        "Sends a message"
        site_encoding = self._site_encoding()
        ##################
        # Create the message
        msg = Message()
        msg.set_payload(body, site_encoding)
        ########################
        # set headers on message
        from_ = self.contact_email()
        if not from_:
            from_ = self._site_email()
        if not from_:
            from_ = 'event@localhost'
        msg['From'] = from_
        msg['Reply-To'] = from_
        msg['To'] = ','.join(to)
        msg['Date'] = self.getNow().rfc822() # needed by some servers
        msg['Subject'] = Header(subject, site_encoding)
        ##################
        # Send the message
        SMTPserver = self._mailhost()
        outbox = smtplib.SMTP(SMTPserver)
        outbox.sendmail(from_, to, msg.as_string())


    #################################
    # Finding free periods for attendees

    def _b2o(self, brains):
        """
        Converts list of brains to objects using unrestrictedTraverse traverse, 
        so that we also can check private objects for busy times.
        """
        objects = []
        for brain in brains:
            try:
                obj = self.unrestrictedTraverse(brain.getPath())
                if not obj is None:
                    objects.append(obj)
            except:
                pass
        return objects

    def eventSearch(self, first_date, last_date):
        "Search for events for all attendees"
        catalog = getToolByName(self, 'portal_catalog')
        catalog = UnrestrictedCatalog(catalog)
        attendee_ids = self.attendee_ids()
        brains = catalog(start={'query': (first_date, last_date), 'range': 'min:max'},
                         attendee_ids=attendee_ids,
                        )
        # Get all events that Ends insidew range
        brains += catalog(end={'query': (first_date, last_date), 'range': 'min:max'},
                          attendee_ids=attendee_ids,
                         )
        # Get all events that Start before this day AND End after this day
        brains += catalog(start={'query': first_date, 'range': 'max'},
                          end={'query': last_date, 'range': 'min'},
                          attendee_ids=attendee_ids,
                         )
        # unique results
        unique = {}
        for brain in brains:
            unique[brain.getRID()] = brain
        return unique.values()

    def _excluded_days(self, weekdays):
        # returns set of excluded days
        return  Set(range(7)) - Set(weekdays)

    def _events2spans(self, events, first_date, last_date):
        spans = []
        for event in events:
            (start, end) = (event.start, event.end)
            if start and end:
                if start < first_date:
                    start = first_date
                if end > last_date:
                    end = last_date
                spans.append( (start,end) )
        return spans

    def _excluded_spans(self, first_date, start_after, end_before, search_period, weekdays):
        "Returns spans for times where there should be no events"
        excluded_days = self._excluded_days(weekdays)
        excluded_spans = []
        for i in range(int(search_period)):
            day = first_date + (i * DAY)
            day_start = day.earliestTime()
            day_end = day_start + 1
            if day.dow() in excluded_days:
                excluded_spans.append((day_start, day_end))
            else:
                first_time = day_start + (HOUR * start_after)
                last_time = day_start + (HOUR * end_before)
                excluded_spans.append( (day_start, first_time) )
                excluded_spans.append( (last_time, day_end) )
        return excluded_spans


###    security.declarePublic('test')
###    def test(self):
###        "dfd"
###        catalog = getToolByName(self, 'portal_catalog')
###        catalog = UnrestrictedCatalog(catalog)
###        attendee_ids = ['mra']
###        brains = catalog(
###            attendee_ids = attendee_ids,
###            portal_type = 'mxmEvent',
###            )
###        result = []
###        for brain in brains:
###            #if brain.attendee_ids:
###            result.append(brain.Title)
###            result.append(repr(brain.attendee_ids))
###            result.append(brain.getPath())
###            result.append('')
###        return '\n'.join(result)

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'findFreePeriods')
    def findFreePeriods(self):
        """
        Returns list of free timespans for the attendees.
        """
        req = self.REQUEST
        # test if search has been comitted
        if not req.get('commit_search', 0):
            return []
        # get search data from request
        unit = req.get('unit', 'hour')
        duration = req.get('duration', 1)
        start_after = req.get('start_after', 8)
        end_before = req.get('end_before', 17)
        search_period = req.get('search_period', 7.0)
        weekdays = req.get('weekdays', [1,2,3,4,5])
        n_results = req.get('n_results', 10)
        # look for open times for every whole hour
        search_resolution = req.get('search_resolution', HOUR)
        # normalize input values
        if unit == 'minute':
            unit_size = MINUTE
        else:
            unit_size = HOUR
        duration_size = duration * unit_size
        # find attendees events
        first_date = self.getNow().earliestTime() # start today
        last_date = first_date + search_period
        events = self.eventSearch(first_date, last_date)
        # find free spans/periods
        spans = self._events2spans(events, first_date, last_date)
        # add first & last to include free times outside events
        spans.append((first_date, first_date))
        spans.append((last_date, last_date))
        # add excluded spans
        spans += self._excluded_spans(first_date, start_after, end_before, search_period, weekdays)
        non_spans = MetaSpans(spans).non_spans()
        # now find the free periods in the non_spans
        strf = '%Y-%m-%d %H:%M:%S'
        free_periods = []
        count = 0
        for non_span in non_spans:
            start, end = non_span
            time_left = end - start
            while (time_left >= duration_size) and (count < n_results):
                free_periods.append( (start, start + duration_size) )
                count += 1
                start += search_resolution
                time_left = end - start
            if count >= n_results:
                break
        return free_periods


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'setMeetingTime')
    def setMeetingTime(self, event_time=None, REQUEST=None):
        "Sets the start-end time for the event"
        # event_time format '%Y%m%dT%H%M%S:%Y%m%dT%H%M%S' 20050512T120000:20050512T130000
        if event_time:
            start_slices = [(0,4), (4,6), (6,8), (9,11), (11,13), (13,15)]
            start_values = [int(event_time[slice(start, end)]) for (start, end) in start_slices]
            start_time = DateTime(*start_values)
            end_slices = [(16,20), (20,22), (22,24), (25,27), (27,29), (29,31)]
            end_values = [int(event_time[slice(start, end)]) for (start, end) in end_slices]
            end_time = DateTime(*end_values)
            self.setStartDate(start_time)
            self.setEndDate(end_time)
        if not REQUEST is None:
            REQUEST.RESPONSE.redirect('%s/base_edit' % self.absolute_url())


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'isCompleted')
    def isCompleted(self):
        "Returns true if the event is completed"
        memberId = self._getMemberId()
        return (
            self.getStatus().upper() == 'COMPLETED'
            or 'COMPLETED' in Set(
                [att.get('PARTSTAT', '').upper() for att in self.getAttendees().values()
                 if att.get(MEMBER_KEY, '') == memberId]
            )
        )


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'isCancelled')
    def isCancelled(self):
        "Returns true if the event is cancelled, or declined for member"
        return (
            self.getStatus().upper() in ('CANCELLED',)
            or 'DECLINED' in Set(
                [att.get('PARTSTAT', '').upper() for att in self.getAttendees().values()
                 if att.get(MEMBER_KEY, '') == self._getMemberId()]
            )
        )

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'isActive')
    def isActive(self):
        "Return true if event is active"
        return not (self.isCompleted() or self.isCancelled())

    def _getMember(self):
        # returns authenticated member
        m_tool = getToolByName(self, 'portal_membership')
        return m_tool.getAuthenticatedMember()

    def _getMemberId(self):
        # returns authenticated member
        member = self._getMember()
        if member:
            return member.getMemberId()

    # start-end setters-getters

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getStart')
    def setStart(self, start):
        "Same as start() method"
        self.setStartDate(start)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getEnd')
    def setEnd(self, end):
        "Same as end() method"
        self.setEndDate(end)


    startDate = None
    endDate = None


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'start')
    def start(self):
        "Same as start() method"
        start = getattr(self.aq_base, 'startDate', None)
        if not start:
            start = self.mxmCalendarTypes_init_start()
        return start

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getStart')
    getStart = start


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'end')
    def end(self):
        "Same as start() method"
        end = getattr(self.aq_base, 'endDate', None)
        if not end:
            end = self.mxmCalendarTypes_init_end()
        return end

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getEnd')
    getEnd = end


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'januaryFirst')
    def januaryFirst(self):
        "Returns January First"
        return self.ZopeTime(2000,1,1)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'aprilFirst')
    def aprilFirst(self):
        "Returns January First"
        return self.ZopeTime(2000,4,1)


    # duration based methods

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getDuration')
    def getDuration(self):
        "Returns the duration or None"
        start = self.start()
        end = self.end()
        if start and end:
            return end - start

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'setDuration')
    def setDuration(self, duration):
        "Sets the duration by setting the end time relative to the start time"
        self.setEnd(self.start() + duration)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getDailyHours')
    def getDailyHours(self):
        """
        If duration <= 1 day, hours is the duration in hours
        If duration > 1 day, hours is the difference between start and
        end times on the start and end days.
        eg. start == 2000-01-01 10:00
              end == 2000-01-02 16:00 ->
            duration == 1.25 days ->
            dailyHours = 0.25 * 24.00 = 6.0 hours
        """
        duration = self.getDuration()
        if not duration or duration < 0.0: # should not happen ...
            return 0.0
        elif duration <= 1.0:
            return duration * 24
        else:
            fract_part, int_part = modf(duration)
            return fract_part * 24

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'moveTo')
    def moveTo(self, new_start):
        "Moves the event to new start time, keeping duration"
        duration = self.getDuration()
        if duration:
            self.setStart(new_start)
            self.setDuration(duration)


    security.declarePrivate('migrate')
    def migrate(self):
        "Migrate to latest version"
        attendees = self.getAttendees()
        for key, attendee in attendees.items():
            print key, attendee
        

class mxmEvent(mxmCalendarType):

    content_icon = 'mxmEvent_icon.gif'
    
    ical_type = 'VEVENT'

    schema = mxmEvent_schema

    actions = (
        {
            'id': 'attendees',
            'name': 'Attendees',
            'action': 'string:${object_url}/mxmCalendarTypes_attendees_form',
            'permissions': (EDIT_CONTENTS_PERMISSION,),
            'visible':1,
        },
        {
            'id': 'freetime',
            'name': 'Free time',
            'action': 'string:${object_url}/mxmCalendarTypes_attendees_freetime',
            'permissions': (EDIT_CONTENTS_PERMISSION,),
            'visible':1,
        },
        {
            'id': 'metadata',
            'name': 'Properties',
            'action': 'string:${object_url}/base_metadata',
            'permissions': (EDIT_CONTENTS_PERMISSION,),
            'visible':0,
        },
    )

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(VIEW_CONTENTS_PERMISSION)



class mxmTodo(mxmCalendarType):

    content_icon = 'mxmTodo_icon.gif'
    
    ical_type = 'VTODO'

    schema = mxmTodo_schema

    actions = (
        {
            'id': 'attendees',
            'name': 'Attendees',
            'action': 'string:${object_url}/mxmCalendarTypes_attendees_form',
            'permissions': (EDIT_CONTENTS_PERMISSION,),
            'visible':1,
        },
        {
            'id': 'timereg',
            'name': 'Time Registration',
            'action': 'string:${object_url}/mxmtodo_timereg',
            'permissions': (EDIT_CONTENTS_PERMISSION,),
            'visible':1,
        },
        {
            'id': 'metadata',
            'name': 'Properties',
            'action': 'string:${object_url}/base_metadata',
            'permissions': (EDIT_CONTENTS_PERMISSION,),
            'visible':0,
        },
    )

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(VIEW_CONTENTS_PERMISSION)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'dateTimeView')
    def dateTimeView(self, date_format='%d/%m/%Y', time_format='%H:%M'):
        "Returns a nicely formatted datetime view"
        end = self.end()
        if end:
            date_time = (date_format, time_format)
            return end.strftime('%s (%s)' % date_time)
        else:
            return ''

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'isActive')
    def isActive(self):
        "Is the todo active"
        return (
            mxmCalendarType.isActive(self)
            and not self.getCompleted()
        )

    # time per attendee

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getRemainingTime')
    def getRemainingTime(self):
        "Return the remaining time in hours"
        return self.getAllocatedTime() - self.getUsedTime()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getEstimatedRemainingTime')
    def getEstimatedRemainingTime(self):
        "Return the estimated remaining time in hours, depending on percent and used time so far"
        percent = self.getPercent()
        used = self.getUsedTime()
        if used > 0 and percent > 0:
            return ((100.0 / percent) * used) - used
        return self.getRemainingTime()

    # total times for all attendees

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getTotalAllocatedTime')
    def getTotalAllocatedTime(self):
        "Returns total time for all attendees."
        return self.getAllocatedTime() * self.nAttendees()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getTotalUsedTime')
    def getTotalUsedTime(self):
        "Returns total time used for all attendees."
        return self.getUsedTime() * self.nAttendees()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getTotalRemainingTime')
    def getTotalRemainingTime(self):
        "Return the remaining time in hours"
        return self.getRemainingTime() * self.nAttendees()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getTotalEstimatedRemainingTime')
    def getTotalEstimatedRemainingTime(self):
        "Return the remaining time in hours"
        return self.getEstimatedRemainingTime() * self.nAttendees()

    # percent

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getPercentUsed')
    def getPercentUsed(self):
        """
        Returns the percent of allocated hours that has been used.
        """
        allocated = self.getAllocatedTime()
        used = self.getUsedTime()
        if allocated > 0.0 and used > 0.0:
            return int(used / allocated * 100.00)
        return 0

    # Time registration methods
    
    def _timeDecode(self, st):
        "Converts a 'time' string to float"
        if not st:
            return 0.0
        if ':' in st:
            hours, minutes = st.split(':')
            if not hours: hours = 0.0
            if not minutes: minutes = 0.0
            return float(hours) + float(minutes)/60.0
        elif ',' in st:
            return float(st.replace(',', '.'))
        return float(st)


    def _getLogTime(self):
        "Returns DateTime object of logtime"
        now = self.ZopeTime()
        pattern = 'timereg_logtime_%s'
        pyear = pattern % 'year'
        pmonth = pattern % 'month'
        pday = pattern % 'day'
        phour = pattern % 'hour'
        pminute = pattern % 'minute'
        # get the data
        req = self.REQUEST
        year = int(req.get(pyear, now.year()))
        month = int(req.get(pmonth, now.month()))
        day = int(req.get(pday, now.day()))
        hour = int(req.get(phour, now.hour()))
        minute = int(req.get(pminute, now.minute()))
        return self.ZopeTime(year, month, day, hour, minute)


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'authenticatedCanRegister')
    def authenticatedCanRegister(self):
        """
        Returns true if the current authenticated user can register time on the 
        task.
        """
        memberId = self._getMemberId()
        return self.isActive() and memberId in self.attendee_ids()




    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'addTimeRegistration')
    def addTimeRegistration(self, memberId, time='0.0', subject='',
                            comment='', logtime=None):
        "Add a time registration to the todo"
        if time > 0.0: # dont register 0 time or less
            reg = {
                'time' : time,
                'subject' : subject,
                'comment' : comment,
                'logtime' : logtime,
                'memberId' : memberId,
            }
            regs = self.getTimeRegistrations()
            regs.append(reg)
            self._time_registrations = regs


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'addTimeRegistrationForAttende')
    def addTimeRegistrationForAttende(self, timereg_time='0.0', timereg_subject='',
                                      timereg_comment='', timereg_logtime=None,
                                      REQUEST=None):
        """
        Add a time registration to the todo. An interface helper.
        """
        if not self.authenticatedCanRegister():
            raise Unauthorized('You are not allowed to register time.')
        timereg_time = self._timeDecode(timereg_time)
        timereg_logtime = self._getLogTime()
        memberId = self._getMemberId()
        self.addTimeRegistration(memberId, time=timereg_time, subject=timereg_subject,
                                 comment=timereg_comment, logtime=timereg_logtime)
        if not REQUEST is None:
            if timereg_time > 0.0:
                psm = '?portal_status_message=Succes! %s hours registered' % timereg_time
            else:
                psm = '?portal_status_message=Error!  %s is not a legal time to register' % timereg_time
            REQUEST.RESPONSE.redirect(self.absolute_url() + psm)




    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'editTimeRegistration')
    def editTimeRegistration(self, id, time=None, subject=None, comment=None, 
                             logtime=None):
        "Edit a single time registration"
        reg = self._time_registrations[id]
        if not time is None:
            reg['time'] = time
        if not subject is None:
            reg['subject'] = subject
        if not comment is None:
            reg['comment'] = comment
        if not logtime is None:
            reg['logtime'] = logtime
        self._p_changed = 1


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'editTimeRegistrationsForAttende')
    def editTimeRegistrationsForAttende(self, loggedTimeIds=None, REQUEST=None):
        """
        Add a time registration to the todo. An interface helper.
        """
        if not self.authenticatedCanRegister():
            raise Unauthorized('You are not allowed to register time.')
        memberId = self._getMemberId()
        if loggedTimeIds is None:
            loggedTimeIds = []
        req = self.REQUEST
        for id in loggedTimeIds:
            time = req['loggedTime_%s' % id]
            time = self._timeDecode(time)
            # check member is only changing own values
            if self._time_registrations[id]['memberId'] != memberId:
                raise Unauthorized('Member can only modify own values.')
            self.editTimeRegistration(id, time=time)
        if not REQUEST is None:
            psm = '?portal_status_message=Succes! Times updated'
            REQUEST.RESPONSE.redirect(self.absolute_url() + psm)


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'editTimeRegistrations')
    def editTimeRegistrations(self, loggedTimeIds=None, REQUEST=None):
        """
        Add a time registration to the todo. An interface helper.
        """
        if loggedTimeIds is None:
            loggedTimeIds = []
        req = self.REQUEST
        for id in loggedTimeIds:
            time = req['loggedTime_%s' % id]
            time = self._timeDecode(time)
            self.editTimeRegistration(id, time=time)
        if not REQUEST is None:
            psm = '?portal_status_message=Succes! Times updated'
            REQUEST.RESPONSE.redirect(self.absolute_url() + '/mxmtodo_timereg' + psm)





    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'deleteTimeRegistrations')
    def deleteTimeRegistrations(self, timeRegIds=None, REQUEST=None):
        "Deletes the time registration with id"
        if timeRegIds is None:
            timeRegIds = []
        regs = self.getTimeRegistrations()
        for id in timeRegIds:
            if regs.has_key(id):
                del regs[id]
        self._time_registrations = regs
        if not REQUEST is None:
            if timeRegIds:
                psm = '?portal_status_message=Succes! %s registrations deleted' % len(timeRegIds)
            else:
                psm = '?portal_status_message=Error! No registrations to delete'
            REQUEST.RESPONSE.redirect(self.absolute_url() + psm)


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'deleteTimeRegistrationsForAttende')
    def deleteTimeRegistrationsForAttende(self, timeRegIds=None, REQUEST=None):
        """
        Deletes the time registration with id.
        """
        if not self.authenticatedCanRegister():
            raise Unauthorized('You are not allowed to register time.')
        self.deleteTimeRegistrations(timeRegIds=timeRegIds, REQUEST=REQUEST)




    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getTimeRegistrations')
    def getTimeRegistrations(self):
        """
        Returns all the time registrations.
        """
        if not hasattr(self, '_time_registrations'):
            self._time_registrations = AutoDict()
        regs = self._time_registrations
        return regs


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getTimeRegistrationsWithTaskTitle')
    def getTimeRegistrationsWithTaskTitle(self):
        """
        Returns all the time registrations. But adds the task title to each 
        registration. Practical in tables.
        """
        regs = self.getTimeRegistrations()
        for reg in regs.values():
            reg['task_title'] = self.title_or_id()
        return regs


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getResolvedRegistrations')
    def getResolvedRegistrations(self):
        """
        Do *not* use this for modifications. This is a user interface helper.
        Returns list of all the time registrations. Not dict!. Fullname and email are 
        resolved. id is inserted also.
        Also sorts by key, if sortkey is set in the request.
        """
        regs = {}
        regs.update(self.getTimeRegistrations())
        for key, reg in regs.items():
            reg['id'] = key
            memberId = reg.get('memberId', '')
            memberProps = self.getMemberProperties(memberId, ['fullname', 'email'])
            reg['fullname'], reg['email'] = memberProps
        regs = regs.values()
        # sort
        sortkey = self.REQUEST.get('sortkey', 'logtime')
        decorated = []
        for r in regs:
            sortval = r.get(sortkey, '')
            if hasattr(sortval, 'lower'):
                sortval = sortval.lower()
            decorated.append((sortval, r))
        decorated.sort()
        regs = [d[-1] for d in decorated]
        return regs


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAuthenticatedsResolvedRegistrations')
    def getAuthenticatedsResolvedRegistrations(self):
        """
        This is a user interface helper.
        Returns list of all the time registrations. Not dict!. Fullname and email are 
        resolved. id is inserted also.
        Also sorts by key, if sortkey is set in the request.
        """
        memberId = self._getMemberId()
        regs = self.getResolvedRegistrations()
        return [reg for reg in regs if reg['memberId'] == memberId]
        
        
    # basic methods

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getTimeRegsByKeyGroup')
    def getTimeRegsByKeyGroup(self, key):
        """
        Returns time regs as dict with list of objects, by key. Eg:
        {'memberId1':[reg1, reg2, reg3], {'memberId2':[reg4, reg5, reg6]}
        """
        regs = self.getTimeRegistrations().values()
        result = {}
        for reg in regs:
            result.setdefault(reg[key], []).append(reg)
        return result

    def _sumTime(self, regs):
        "Sums time from registrations"
        return sum([float(reg.get('time', 0.0)) for reg in regs])

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getTotalTimesByKeyGroup')
    def getTotalTimesByKeyGroup(self, key):
        "Returns total time grouped by key"
        regGroups = self.getTimeRegsByKeyGroup(key)
        result = {}
        for k, regs in regGroups.items():
            result[k] = self._sumTime(regs)
        return result

    #

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getMembersTimeRegistrations')
    def getMembersTimeRegistrations(self, memberId):
        "Returns the time registrations for authenticated member"
        regs = self.getTimeRegsByKeyGroup('memberId')
        memberRegs = regs.get(self._getMemberId(), [])
        return memberRegs

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getmembersTotalTime')
    def getmembersTotalTime(self, memberId):
        "Returns total time by memberId"
        return self.getTotalTimesByKeyGroup('memberId').get(memberId, 0.0)


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAuthenticatedsTimeRegistrations')
    def getAuthenticatedsTimeRegistrations(self):
        "Returns the time registrations for authenticated member"
        return self.getMembersTimeRegistrations(self._getMemberId())

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAuthenticatedsTotalTime')
    def getAuthenticatedsTotalTime(self):
        "Returns total time for authenticated member"
        return self.getmembersTotalTime(self._getMemberId())

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getTotalTime')
    def getTotalTime(self, start=None, end=None):
        "Returns the time in hours, used by all members on an event"
        regs = self.getTimeRegistrations().values()
        if start and end:
            regs = [reg for reg in regs if start <= reg['logtime'] <= end]
        return self._sumTime(regs)


    # backwards comp. deprecated methods!
    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getUsedTime')
    getUsedTime = getTotalTime

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getUsedTimeForMember')
    getUsedTimeForMember = getmembersTotalTime


    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getMemberProperties')
    def getMemberProperties(self, memberId, properties):
        "Returns one or more members properties"
        if isinstance(properties, StringTypes):
            properties = [properties]
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getMemberById(memberId)
        if member is None: # just return empty strings if member has been deleted
            vals = ['']*len(properties)
        else:
            vals = [member.getProperty(property, '') for property in properties]
        if len(vals) == 1:
            return vals[0]
        else:
            return vals
        
    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getMemberFullname')
    def getMemberFullname(self, memberId):
        return self.getMemberProperties(memberId, 'fullname')




class mxmJournal(mxmCalendarType):

    content_icon = 'mxmJournal_icon.gif'

    ical_type = 'VJOURNAL'

    schema = mxmJournal_schema

    actions = (
        {
            'id': 'attendees',
            'name': 'Attendees',
            'action': 'string:${object_url}/mxmCalendarTypes_attendees_form',
            'permissions': (EDIT_CONTENTS_PERMISSION,),
            'visible':1,
        },
        {
            'id': 'metadata',
            'name': 'Properties',
            'action': 'string:${object_url}/base_metadata',
            'permissions': (EDIT_CONTENTS_PERMISSION,),
            'visible':0,
        },
    )
    
    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(VIEW_CONTENTS_PERMISSION)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'dateTimeView')
    def dateTimeView(self, date_format='%d/%m/%Y', time_format='%H:%M'):
        "Returns a nicely formatted datetime view"
        start = self.start() or self.getNow()
        date_time = (date_format, time_format)
        return start.strftime('%s %s' % date_time)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'end')
    def end(self):
        "This makes it easier to write general code for all events"
        return None




setDefaultRoles(ADD_CONTENTS_PERMISSION,  ('Manager', 'Owner',))
setDefaultRoles(EDIT_CONTENTS_PERMISSION, ('Manager', 'Owner',))


registerType(mxmEvent, PROJECTNAME)
registerType(mxmTodo, PROJECTNAME)
registerType(mxmJournal, PROJECTNAME)

