# -*- coding: ISO-8859-1 -*-

from Products.Archetypes.Marshall import RFC822Marshaller

from Products.Archetypes.public import BaseSchema, BaseFolderSchema, Schema
from Products.Archetypes.public import StringField, StringWidget
from Products.Archetypes.public import TextField, TextAreaWidget
from Products.Archetypes.public import VisualWidget

from Products.Archetypes.public import IntegerField, IntegerWidget
from Products.Archetypes.public import FloatField, DecimalWidget
from Products.Archetypes.public import BooleanField, BooleanWidget
from Products.Archetypes.public import LinesField, LinesWidget
from Products.Archetypes.public import SelectionWidget, MultiSelectionWidget,\
                                       DisplayList
from Products.Archetypes.public import DateTimeField, CalendarWidget
from Products.Archetypes.public import ReferenceField, ReferenceWidget

TRANSPARENCY = ('Opaque', 'Transparent',)
PRIORITY = (('0', 'Undefined'), ('1', 'High'), ('5', 'Medium'), ('9', 'Low'),)

# proposed iCalendar categories
EVENT_CATEGORIES = (
    'Anniversary', 'Appointment', 'Business', 'Education', 'Holiday',
    'Meeting', 'Miscellaneous', 'Non-Working Hours', 'Not In Office',
    'Personal', 'Phone Call', 'Sick Day', 'Special Occasion', 'Travel',
    'Vacation',
)
EVENT_STATUS = ('Tentative', 'Confirmed', 'Cancelled',)



mxmEvent_schema = BaseSchema + Schema(
    (

        StringField('title',
            widget = StringWidget(
                label = 'Title/summary',
                description = 'Title/summary of the content',
                size = 60,
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_title",
                description_msgid = "help_title",
            ),
            accessor ='Title',
            required = 1,
            searchable = 1,
        ),

        TextField('description',
            widget = TextAreaWidget(
                label = 'Description',
                description = 'A short description of the content',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_description",
                description_msgid = "help_description",
            ),
            accessor='Description',
            searchable = 1,
        ),

        DateTimeField('startDate',
            widget = CalendarWidget(
                label = 'Start',
                description = 'Start time of event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_startDate",
                description_msgid = "help_startDate",
            ),
            default_method = 'mxmCalendarTypes_init_start',
#            default_method = 'januaryFirst',
            accessor='start',
            required = 1,
        ),

        DateTimeField('endDate',
            widget = CalendarWidget(
                label = 'End',
                description = 'End time of event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_endDate",
                description_msgid = "help_endDate",
            ),
            default_method = 'mxmCalendarTypes_init_end',
#            default_method = 'aprilFirst',
            accessor='end',
            required = 1,
        ),

        StringField('location',
            widget = StringWidget(
                label = 'Location',
                description = 'Location of the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_location",
                description_msgid = "help_location",
            ),
            searchable = 1,
        ),

        LinesField('subject',
            widget = MultiSelectionWidget(
                label = 'Categories',
                description = 'Categories/Subject',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_subject",
                description_msgid = "help_subject",
            ),
            accessor ='Subject',
            vocabulary = 'allowedSubjectsDisplayList',
            default = (),
        ),

        StringField('eventUrl',
            widget = StringWidget(
                label = 'Url',
                description = 'Link (URL) to the event',
                size = 60,
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_eventUrl",
                description_msgid = "help_eventUrl",
            ),
            accessor='event_url',
            searchable = 1,
        ),

        # value between 0-9
        IntegerField('priority',
            widget = SelectionWidget(
                label = 'Priority',
                description = 'Importance if the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_priority",
                description_msgid = "help_priority",
            ),
            default = 0,
            vocabulary = DisplayList(PRIORITY),
        ),

        StringField('contactName',
            widget = StringWidget(
                label = 'Organizer',
                description = 'Organiser of the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contactName",
                description_msgid = "help_contactName",
            ),
            default_method = 'getAuthenticatedsFullname',
            accessor='contact_name',
            searchable = 1,
        ),

        StringField('contactEmail',
            widget = StringWidget(
                label = 'Organizer e-mail',
                description = 'Email address of organizer of the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contactEmail",
                description_msgid = "help_contactEmail",
            ),
            default_method = 'getAuthenticatedsEmail',
            accessor='contact_email',
            searchable = 1,
        ),

        StringField('contactRole',
            widget = SelectionWidget(
                label = 'Organizers role',
                description = 'The organizers role in the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contactRole",
                description_msgid = "help_contactRole",
            ),
            default = '',
            vocabulary = 'possibleRolesItems',
        ),

        TextField('contact',
            widget = TextAreaWidget(
                label = 'Contact',
                description = 'Other contact info',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contact",
                description_msgid = "help_contact",
            ),
            accessor='contact_phone',
            searchable = 1,
        ),

        # workflow instead?
        StringField('status',
            widget = SelectionWidget(
                label = 'Global event status',
                description = 'Global status of the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_status",
                description_msgid = "help_status",
            ),
            vocabulary = EVENT_STATUS,
        ),

        StringField('transparency',
            widget = SelectionWidget(
                label = 'Transparency',
                description = 'If an event is transparent, other events can be booked at the same time',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_transparency",
                description_msgid = "help_transparency",
            ),
            vocabulary = TRANSPARENCY,
            default = TRANSPARENCY[0],
        ),

        BooleanField('emailNotifications',
            widget = BooleanWidget(
                label = 'Email notifications',
                description = 'Automatically send email notifications on attendees status change',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_emailNotifications",
                description_msgid = "help_emailNotifications",
            ),
        ),

    ),
    marshall=RFC822Marshaller(),
)

# setI18n(mxmEvent_schema, default_domain)


TODO_CATEGORIES = ()
TODO_STATUS = ('Needs action', 'Completed', 'In-Process', 'Cancelled',)
mxmTodo_schema = BaseSchema + Schema(
    (

        StringField('title',
            widget = StringWidget(
                label = 'Title/summary',
                description = 'Title/summary of the todo',
                size = 60,
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_title",
                description_msgid = "help_title",
            ),
            accessor ='Title',
            required = 1,
            searchable = 1,
        ),

        TextField('description',
            widget = TextAreaWidget(
                label = 'Description',
                description = 'A short description of the todo',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_description",
                description_msgid = "help_description",
            ),
            accessor='Description',
            searchable = 1,
        ),

        DateTimeField('startDate',
            widget = CalendarWidget(
                label = 'Start',
                description = 'Start time of todo',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_startDate",
                description_msgid = "help_startDate",
            ),
            default_method = 'getNow',
            accessor='start',
        ),

        DateTimeField('endDate',
            widget = CalendarWidget(
                label = 'Due',
                description = 'Due time of event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_due",
                description_msgid = "help_due",
            ),
            default_method = 'getInAnHour',
            accessor='end',
        ),




        FloatField('allocatedTime',
            widget = DecimalWidget(
                label = 'Allocated time',
                description = 'Total time allocated to the event.',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_allocatedTime",
                description_msgid = "help_allocatedTime",
            ),
            default=0.0,
        ),


        # value between 0-100
        IntegerField('percent',
            widget = IntegerWidget(
                label = 'Percent',
                description = 'Percent completed',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_percent",
                description_msgid = "help_percent",
            ),
            default = 0,
        ),




        DateTimeField('completed',
            widget = CalendarWidget(
                label = 'Completed',
                description = 'Actual completion date',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_completed",
                description_msgid = "help_completed",
            ),
        ),

        StringField('location',
            widget = StringWidget(
                label = 'Location',
                description = 'Location of the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_location",
                description_msgid = "help_location",
            ),
            searchable = 1,
        ),

        LinesField('subject',
            widget = MultiSelectionWidget(
                label = 'Categories',
                description = 'Categories/Subject',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_subject",
                description_msgid = "help_subject",
            ),
            accessor ='Subject',
            vocabulary = 'allowedSubjectsDisplayList',
            default = (),
        ),

        StringField('eventUrl',
            widget = StringWidget(
                label = 'Url',
                description = 'Link (URL) to the event',
                size = 60,
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_eventUrl",
                description_msgid = "help_eventUrl",
            ),
            accessor='event_url',
            searchable = 1,
        ),

        # value between 0-9
        IntegerField('priority',
            widget = SelectionWidget(
                label = 'Priority',
                description = 'Importance of the todo',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_priority",
                description_msgid = "help_priority",
            ),
            default = 0,
            vocabulary = DisplayList(PRIORITY),
        ),

        StringField('contactName',
            widget = StringWidget(
                label = 'Organizer',
                description = 'Organiser of the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contactName",
                description_msgid = "help_contactName",
            ),
            default_method = 'getAuthenticatedsFullname',
            accessor='contact_name',
            searchable = 1,
        ),

        StringField('contactEmail',
            widget = StringWidget(
                label = 'Organizer e-mail',
                description = 'Email address of organizer of the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contactEmail",
                description_msgid = "help_contactEmail",
            ),
            default_method = 'getAuthenticatedsEmail',
            accessor='contact_email',
            searchable = 1,
        ),

        StringField('contactRole',
            widget = SelectionWidget(
                label = 'Organizers role',
                description = 'The organizers role in the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contactRole",
                description_msgid = "help_contactRole",
            ),
            default = '',
            vocabulary = 'possibleRolesItems',
        ),

        TextField('contact',
            widget = TextAreaWidget(
                label = 'Contact',
                description = 'Other contact info',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contact",
                description_msgid = "help_contact",
            ),
            accessor='contact_phone',
            searchable = 1,
        ),

        # workflow instead?
        StringField('status',
            widget = SelectionWidget(
                label = 'Todo status',
                description = 'Status of the todo',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_status",
                description_msgid = "help_status",
            ),
            vocabulary = TODO_STATUS,
#            default = TODO_STATUS[0],
        ),

        BooleanField('emailNotifications',
            widget = BooleanWidget(
                label = 'Email notifications',
                description = 'Automatically send email notifications on attendees status change',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_emailNotifications",
                description_msgid = "help_emailNotifications",
            ),
        ),

    ),
    marshall=RFC822Marshaller(),
)


# this looks a lot like a blog item ...
JOURNAL_CATEGORIES = ()
JOURNAL_STATUS = ('Draft', 'Final', 'Cancelled',)
mxmJournal_schema = BaseSchema + Schema(
    (

        StringField('title',
            widget = StringWidget(
                label = 'Title/summary',
                description = 'Title/summary of the journal',
                size = 60,
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_title",
                description_msgid = "help_title",
            ),
            accessor ='Title',
            required = 1,
            searchable = 1,
        ),

        TextField('description',
            widget = TextAreaWidget(
                label = 'Description',
                description = 'A short description of the journal',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_description",
                description_msgid = "help_description",
            ),
            accessor='Description',
            searchable = 1,
        ),

        DateTimeField('startDate',
            widget = CalendarWidget(
                label = 'Start',
                description = 'Start time of event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_startDate",
                description_msgid = "help_startDate",
            ),
            default_method = 'getNow',
            accessor='start',
        ),

        LinesField('subject',
            widget = MultiSelectionWidget(
                label = 'Categories',
                description = 'Journal categories/Subject',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_subject",
                description_msgid = "help_subject",
            ),
            accessor ='Subject',
            vocabulary = 'allowedSubjectsDisplayList',
            default = (),
        ),

        StringField('eventUrl',
            widget = StringWidget(
                label = 'Url',
                description = 'Link (URL) to the event',
                size = 60,
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_eventUrl",
                description_msgid = "help_eventUrl",
            ),
            accessor='event_url',
            searchable = 1,
        ),

        StringField('contactName',
            widget = StringWidget(
                label = 'Organizer',
                description = 'Organiser of the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contactName",
                description_msgid = "help_contactName",
            ),
            default_method = 'getAuthenticatedsFullname',
            accessor='contact_name',
            searchable = 1,
        ),

        StringField('contactEmail',
            widget = StringWidget(
                label = 'Organizer e-mail',
                description = 'Email address of organizer of the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contactEmail",
                description_msgid = "help_contactEmail",
            ),
            default_method = 'getAuthenticatedsEmail',
            accessor='contact_email',
            searchable = 1,
        ),

        StringField('contactRole',
            widget = SelectionWidget(
                label = 'Organizers role',
                description = 'The organizers role in the event',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contactRole",
                description_msgid = "help_contactRole",
            ),
            default = '',
            vocabulary = 'possibleRolesItems',
        ),

        TextField('contact',
            widget = TextAreaWidget(
                label = 'Contact',
                description = 'Other contact info',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_contact",
                description_msgid = "help_contact",
            ),
            accessor='contact_phone',
            searchable = 1,
        ),

        # workflow instead?
        StringField('status',
            widget = SelectionWidget(
                label = 'Journal status',
                description = 'Status of the journal',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_status",
                description_msgid = "help_status",
            ),
            vocabulary = JOURNAL_STATUS,
#            default = JOURNAL_STATUS[0],
        ),

        BooleanField('emailNotifications',
            widget = BooleanWidget(
                label = 'Email notifications',
                description = 'Automatically send email notifications on attendees status change',
                i18n_domain = "mxmcalendartypes",
                label_msgid = "label_emailNotifications",
                description_msgid = "help_emailNotifications",
            ),
        ),

    ),
    marshall=RFC822Marshaller(),
)

