## Script (Python) "mxmCalendarTypes_notifications"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=organizer, attendees
##title=Calendar notifications
##

# Returns (emails, subject, body) for people that should receive a
# notification on the event
# 'organizer' is email address of organizer
# 'attendees' is list of dicts,
# you can use it to check for individual status
# [
# {
#    ID         : 'smith',
#    EMAIL      : 'smith@example.com',
#    'ROLE'     : 'REQ-PARTICIPANT',
#    'PARTSTAT' : 'ACCEPTED',
# },
# ]

EMAIL = 'X-MXM-EMAIL'
ID = 'X-MXM-MEMBER-ID' # Plone member id, if any

# find participation status
partstat = {}
for att in attendees:
    partstat[att.get('PARTSTAT', '').upper()] = None
stata = partstat.keys()
# true if alle attendees has replied
all_replied = ('TENTATIVE' not in stata) and ('' not in stata)
# true if all attendees has accepted the event
all_accepted = (stata == ['ACCEPTED'])

# if not everyone has replied, do nothing.
# if all accepts, send accepted email to organizer and attendees
# if not all has accepted, send mail to organizer only.
if context.getEmailNotifications():
    if all_replied:
        recepients = {}
        if all_accepted: # send to everyone
            if organizer:
                recepients[organizer] = None
            for att in attendees:
                if att.get(EMAIL, ''):
                    recepients[att[EMAIL]] = None
            subject = '[ACCEPTED] %s' % context.title_or_id()
            body = 'All attendees has accepted the event at:'
            body += context.absolute_url()
        else: # send only to organizer, to enable organizing a new event.
            subject = '[DECLINED] %s' % context.title_or_id()
            body = 'All attendees has set their status, but not all has accepted the event at:'
            body += context.absolute_url()
            if organizer:
                recepients[organizer] = None
        return (recepients.keys(), subject, body)

# no recepients should be notified
return ([], '', '')
